# -*- coding: utf-8 -*-
# notebook_utils.py

import json
import os
import urllib

import ipykernel

try:
    from notebook import notebookapp
except ImportError:
    from notebook import app as notebookapp


def currentNBpath():
    """Returns the absolute path of the Notebook or None if it cannot be determined
    NOTE: works only for *Jupyter Notebook* (not Jupyter Lab)
    and when the security is token-based or there is also no password.
    """
    connection_file = os.path.basename(ipykernel.get_connection_file())
    kernel_id = connection_file.split("-", 1)[1].split(".")[0]

    for srv in notebookapp.list_running_servers():
        try:
            if srv["token"] == "" and not srv["password"]:  # No token and no password, ahem...
                req = urllib.request.urlopen(srv["url"] + "api/sessions")
            else:
                req = urllib.request.urlopen(srv["url"] + "api/sessions?token=" + srv["token"])
            sessions = json.load(req)
            for sess in sessions:
                if sess["kernel"]["id"] == kernel_id:
                    return os.path.join(srv["notebook_dir"], sess["notebook"]["path"])
        except OSError:
            pass  # There may be stale entries in the runtime directory
    return None



# -*- coding: utf-8 -*-
# readdata.py

import os

import pandas as pd


def readdata(fn, q_range=None, read_csv_args=None, print_filename=True):
    """Read a datafile pandas Dataframe
    extract a file_name
    select q-range: q_min <= q <= q_max
    """
    if print_filename:
        print(f"Reading file '{fn}'")
    if read_csv_args is None:
        read_csv_args = dict()
    if "sep" not in read_csv_args:
        read_csv_args.update(sep=r"\s+")
    if "names" not in read_csv_args:
        read_csv_args.update(names=("q", "I", "e"))
    if "index_col" not in read_csv_args:
        read_csv_args.update(index_col=False)
    # print("f_read_data, read_csv_args:", read_csv_args) # for debugging

    _, file_ext = os.path.splitext(fn)
    if file_ext.lower() == ".pdh":  # for PDH files
        nrows = pd.read_csv(
            fn,
            skiprows=2,
            nrows=1,
            usecols=[
                0,
            ],
            sep=r"\s+",
            header=None,
        ).values[0, 0]
        read_csv_args.update(skiprows=5, nrows=nrows)
    df_data = pd.read_csv(fn, **read_csv_args)

    # select q-range
    if q_range is not None:
        q_min, q_max = q_range
        df_data = df_data[(df_data.q > q_min) & (df_data.q < q_max)]

    file_name = os.path.basename(fn).split("[")[0]
    return df_data, file_name



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Updates any files with templates in <project root>/ci/templates directory by
# running 'python3 ci/update.py --no-env'
# Typically, github workflow files are generated with the help of tox and its config.
import os
import pathlib
import subprocess
import sys

import jinja2
import toml
import yaml

base_path: pathlib.Path = pathlib.Path(__file__).resolve().parent.parent
templates_path = base_path / "ci" / "templates"


def check_call(args):
    print("+", *args)
    subprocess.check_call(args)


def exec_in_env():
    env_path = base_path / ".tox" / "bootstrap"
    if sys.platform == "win32":
        bin_path = env_path / "Scripts"
    else:
        bin_path = env_path / "bin"
    if not env_path.exists():
        import subprocess

        print("Making bootstrap env in: {0} ...".format(env_path))
        try:
            check_call([sys.executable, "-m", "venv", env_path])
        except subprocess.CalledProcessError:
            try:
                check_call([sys.executable, "-m", "virtualenv", env_path])
            except subprocess.CalledProcessError:
                check_call(["virtualenv", env_path])
        print("Installing `jinja2` into bootstrap environment...")
        check_call([bin_path / "pip", "install", "jinja2", "tox"])
    python_executable = bin_path / "python"
    if not python_executable.exists():
        python_executable = python_executable.with_suffix(".exe")

    print("Re-executing with: {0}".format(python_executable))
    print("+ exec", python_executable, __file__, "--no-env")
    os.execv(python_executable, [python_executable, __file__, "--no-env"])


def loadYaml(filepath: pathlib.Path):
    if not filepath.is_file():
        return {}
    with open(filepath, "r") as fh:
        try:
            return yaml.safe_load(fh)
        except yaml.YAMLError as exc:
            print(exc)
            return {}


def main():
    print("Project path: {0}".format(base_path))
    # use pyproject.toml as source for paths and urls
    project_meta = toml.load(base_path / "pyproject.toml")
    # use cookiecutter cfg to file template params not rendered yet
    cc = loadYaml(base_path / ".cookiecutterrc")
    pypi_host = cc.get("default_context", {}).get("pypi_host", "").split(".")[:-1]

    jinja = jinja2.Environment(
        loader=jinja2.FileSystemLoader(str(templates_path)),
        trim_blocks=True,
        lstrip_blocks=True,
        keep_trailing_newline=True,
    )

    tox_environments = [
        line.strip()
        # 'tox' need not be installed globally, but must be importable
        # by the Python that is running this script.
        for line in subprocess.check_output(
            [sys.executable, "-m", "tox", "--listenvs"], universal_newlines=True
        ).splitlines()
    ]
    # add a version number to the generic Python3 name (just 'py') for templates to build ok
    tox_environments = [
        (line if len(line) > 2 else line + "".join(sys.version.split(".")[:2]))
        for line in tox_environments
        if line.startswith("py")
    ]

    for template in templates_path.rglob("*"):
        if template.is_file() and template.name != ".DS_Store":
            template_path = str(template.relative_to(templates_path))
            destination = base_path / template_path
            destination.parent.mkdir(parents=True, exist_ok=True)
            destination.write_text(
                jinja.get_template(template_path).render(
                    tox_environments=tox_environments,
                    docs_url=project_meta["project"]["urls"]["documentation"],
                    cov_report_path=project_meta["tool"]["coverage"]["report"]["path"],
                    # Python version to use for general tasks: docs (when tox did not set one)
                    py_ver=".".join(sys.version.split(".")[:2]),
                    pypi_token=(
                        "_".join(pypi_host + ["token"]).upper()
                        if len(pypi_host)
                        else "TEST_PYPI_TOKEN"
                    ),
                    pypi_repo="".join(pypi_host) if len(pypi_host) else "testpypi",
                )
            )
            print("Wrote {}".format(template_path))
    print("DONE.")


if __name__ == "__main__":
    args = sys.argv[1:]
    if args == ["--no-env"]:
        main()
    elif not args:
        exec_in_env()
    else:
        print("Unexpected arguments {0}".format(args), file=sys.stderr)
        sys.exit(1)



# -*- coding: utf-8 -*-
# __init__.py

__version__ = "1.0.0"

from .binning import reBin
from .git import checkRepo, isNBstripoutActivated, isNBstripoutInstalled, isRepo
from .notebook_utils import currentNBpath
from .readdata import readdata
from .readdata import readdata as readPDH
from .utils import setLocaleUTF8
from .widgets import PathSelector, showBoolStatus

setLocaleUTF8()

# vim: set ts=4 sts=4 sw=4 tw=0:



# -*- coding: utf-8 -*-
# utils.py

import contextlib
import copy
import itertools
import locale
import os
import platform
import subprocess
import sys
from pathlib import Path

import numpy as np

indent = "    "


def setLocaleUTF8():
    """Fix the Jupyter locale which is not UTF-8 by default on some systems (older Windows?)."""
    locOld = locale.getpreferredencoding(False).lower()

    def getpreferredencoding(do_setlocale=True):
        return "utf-8"

    locale.getpreferredencoding = getpreferredencoding
    locNew = locale.getpreferredencoding(False)
    if locOld != locNew:
        print(f"Updated locale from {locOld} -> {locNew}.")


def isLinux():
    return platform.system().lower() in "linux"


def isMac():
    return platform.system().lower() in "darwin"


def isWindows():
    return platform.system().lower() in "windows"


def isList(obj):
    """Return true if the provided object is list-like including a numpy array
    but not a string.

    >>> isList([1, 2, 'a'])
    True
    >>> isList(tuple((1, 2, 'a')))
    True
    >>> import numpy
    >>> isList(numpy.arange(5))
    True
    >>> isList("dummy")
    False
    >>> isList(None)
    False
    """
    return isinstance(obj, (list, tuple, np.ndarray))


def shortenWinPath(path):
    if not isWindows():
        return path
    import win32api

    return win32api.GetShortPathName(path)


def appendToPATH(parentPath, subdirs=None):
    """Adds the given path with each subdirectory to the PATH environment variable."""
    if not os.path.isdir(parentPath):
        return  # nothing to do
    if subdirs is None:
        subdirs = ["."]
    for path in subdirs:
        path = os.path.realpath(os.path.join(parentPath, *path.split("/")))
        print(indent, path, "\t[{}]".format(os.path.isdir(path)))
        if path in os.environ["PATH"]:
            continue
        os.environ["PATH"] += ";" + path


def checkWinFor7z():
    """Extend the PATH environment variable for access to the 7-zip executable."""
    if not isWindows():
        return  # tests below are intended for Windows
    sevenzippath = r"C:\Program Files\7-Zip"
    if not os.path.isdir(sevenzippath):
        print(
            "7-Zip not found in '{}'.\n".format(sevenzippath)
            + "7-Zip is required for managing data files and results!."
        )
        return
    print("Adding the following directory to $PATH:")
    appendToPATH(sevenzippath)
    print("\nUpdated PATH:")
    for path in os.environ["PATH"].split(";"):
        print(indent, path)


def extract7z(fn, workdir=None):
    assert os.path.isfile(os.path.join(workdir, fn)), "Provided 7z archive '{}' not found!".format(
        fn
    )
    print("Extracting archived McDLS results:")
    proc = subprocess.run(
        ["7z", "x", fn],
        cwd=workdir,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
    )
    print(proc.stdout.decode(errors="ignore"))
    if len(proc.stderr):
        print("## stderr:\n", proc.stderr.decode(errors="ignore"))


# https://stackoverflow.com/a/13847807
@contextlib.contextmanager
def pushd(new_dir):
    previous_dir = os.getcwd()
    os.chdir(new_dir)
    yield
    os.chdir(previous_dir)


def setPackage(globalsdict):
    """Sets the current directory of the notebook as python package
    to make relative module imports work.
    Usage: `setPackage(globals())`"""
    path = Path().resolve()
    searchpath = str(path.parent)
    if searchpath not in sys.path:
        sys.path.insert(0, searchpath)
    globalsdict["__package__"] = path.name
    globalsdict["__name__"] = path.name
    print(f"Setting the current directory as package '{path.name}':\n  {path}.")


def grouper(iterable, n, fillvalue=None):
    """Returns an iterator over a list of tuples (grouping) for a given flat iterable."""
    args = [iter(iterable)] * n
    return itertools.zip_longest(*args, fillvalue=fillvalue)


def fmtErr(val, std, precision=2, width=None):
    """Formats a given value and its std. deviation to physics notation, e.g. '1.23(4)'."""
    if width is None:
        width = ""
    fmt = "{:" + str(width) + "." + str(precision) + "f}({:.0f})"
    # print("fmtErr val:", val, "std:", std)
    return fmt.format(val, std * 10 ** (precision))


def updatedDict(d, key, value):
    """Implements the \\|= operator for dict in Python version <3.9."""
    dd = copy.copy(d)
    dd[key] = value
    return dd
